﻿using UnityEngine;
using System.Collections;

public class Enemy_Spawner : MonoBehaviour {

	public GameObject enemyPrefab;
	public float width = 10f;
	public float height = 5f;
	public float speed;
	public float x_max;
	public float x_min;
	public float spawnDelay;
	
	private bool movingRight = true;
	
	//Initialize
	void Start () {
	
		float distanceToCamera = transform.position.z - Camera.main.transform.position.z;
		
		Vector3 leftBoundary = Camera.main.ViewportToWorldPoint(new Vector3(0,0, distanceToCamera));
		Vector3 rightBoundary = Camera.main.ViewportToWorldPoint(new Vector3(1,0, distanceToCamera));
		
		x_max = rightBoundary.x;
		x_min = leftBoundary.x;
		
		SpawnUntilFull();
		
	}

	void SpawnUntilFull(){
		Transform freePosition = NextFreePosition();
		if(freePosition){
			GameObject enemy = Instantiate(enemyPrefab, freePosition.position, Quaternion.identity) as GameObject;
			enemy.transform.parent = freePosition;
			
		}
		if(NextFreePosition()){
			Invoke ("SpawnUntilFull", spawnDelay);
		}

	}
	
	public void OnDrawGizmos(){
		Gizmos.DrawWireCube (transform.position, new Vector3(width, height));
	}
	
	// Update
	void Update () {
		if(movingRight){
			transform.position += Vector3.right * speed * Time.deltaTime;
		}else{
			transform.position += Vector3.left * speed * Time.deltaTime;
		}
		float rightBoundaryOfFormation = transform.position.x + (0.5f*width);
		float leftBoundaryOfFormation = transform.position.x - (0.5f*width);
		if (leftBoundaryOfFormation < x_min){
			movingRight = true;
		}else if(rightBoundaryOfFormation > x_max){
			movingRight = false;
		}
		if (AllMembersDead()){
			Debug.Log ("Empty Formation");
			SpawnUntilFull();
		}
	}
	
	Transform NextFreePosition(){
		foreach(Transform childPositionGameObject in transform){
			if (childPositionGameObject.childCount == 0){
				return childPositionGameObject;
			}
		}
		return null;	
	}
	
	bool AllMembersDead(){
		foreach(Transform childPositionGameObject in transform){
			if (childPositionGameObject.childCount > 0){
				return false;
			}
		}
		return true;
	}
}
